% CREATEINPFROMSPLINE.M
% Greg Johnson & Matt Fox
% April (almost) 2004
%
% Reads original (Patran generated) INP file, breaks into two parts, and
% adds the displacement conditions as read from the spline.mat file.  Works
% on the INP file found in the current directory (clever detection to be
% done by Matt) and the main INP file generated (to be called by abaqus) is
% suffixed as ABQ.inp





% path should include /cygwin/bin for grep and sed to work

elset='rivet';
material='aluminium';

infile='rivet.inp';
outfile='rivetABQ.inp';
meshfile='rivetmesh.inp';
disp('Grabbing original mesh...');
% grab everything up to the start of the steps
%cmd=['!sed -n -e "/*node/,/*elset/p" ' infile ' > ' meshfile];
% this sed command extracts all the nodes and elements, and also creates 
% an element set to be 'rivet'.  Sed is cool...

% works with ABQCAE generated INP
disp('Did you really use CAE to build this???')
cmd=['!sed -ne "s/*Element.*/\0,elset=rivet/"   -ne "/** region/Iq" -ne "/*node/I,$p" ' infile ' > ' meshfile];
% works with PATRAN generated INP
% disp('Are you really using Patran???')
% cmd=['!sed -ne "/*solid/Iq" -ne "/*node/I,$p" ' infile ' > ' meshfile];


eval(cmd);

% read the nodes to identify the Top set...
disp('Reading nodes');
nodes=read_inp(meshfile);
nodes.topnodes=find(nodes.z==max(nodes.z));

% Left and Right locking nodes assumed to be at extremes of dataset...
[tmp,nodes.left]=min(nodes.x(nodes.topnodes));
[tmp,nodes.right]=max(nodes.x(nodes.topnodes));


%nodes.left=17179;
%nodes.right=17199;

load splineA;
%load splineB;


surfA=fnval(splineA,[nodes.x(nodes.topnodes); nodes.y(nodes.topnodes)]);
%
%surfB=fnval(splineB,[nodes.x(nodes.topnodes)*10]);

% combine surfaces into surfC
%...


fid=fopen(outfile,'wb');
fprintf(fid,'*HEADING\n');
fprintf(fid,'RivetPB, rivet\n');
fprintf(fid,'**\n');
fprintf(fid,'*include,input=rivetmesh.inp\n');
fprintf(fid,'**\n');

fprintf(fid,'*solid section, elset=%s, material=%s\n',elset,material);
fprintf(fid,'*material, name=%s\n',material);
fprintf(fid,'*elastic, type=iso\n');
fprintf(fid,' 7.1e9, 0.29\n');
fprintf(fid,'**\n');

fprintf(fid,'** STEP (autogenerated by Matlab)\n');
fprintf(fid,'*step, perturbation, name=step1\n');
fprintf(fid,'*static\n');
fprintf(fid,'*boundary, op=new\n');
fprintf(fid,'%d,  2,,  0.\n',nodes.left);
fprintf(fid,'%d, 1,2, 0.\n',nodes.right);
fprintf(fid,'*boundary, op=new\n');

disp('Looping through top nodes');
%q=1:length(nodes.topnodes);
%w=((q-(length(nodes.topnodes)/2))/50).^2;
fprintf(fid,'%d, 3,, %f\n',[nodes.topnodes; surfA]);

fprintf(fid,'**\n');
fprintf(fid,'**\n');
fprintf(fid,'**\n');

fprintf(fid,'*output, field, variable=preselect\n');
fprintf(fid,'*file format, ascii\n');
fprintf(fid,'*node print, freq=0\n');
fprintf(fid,'*node file\n');
fprintf(fid,'U,RF\n');
fprintf(fid,'*el print, pos=integ, freq=0\n');
fprintf(fid,'*el print, pos=averaged at nodes, freq=1\n');
fprintf(fid,'S11, S22, S33\n');
fprintf(fid,'*el file, pos=averaged at nodes\n');
fprintf(fid,'S\n');
fprintf(fid,'*end step\n');
fclose(fid);

disp('Closing file');