function [varargout]=gen_outline(varargin)
%% GEN_OUTLINE takes cmm boundary data and produces either a Patran Neutral
% file or a .poly file (for use with Triangle) or a DXF.
%
% Modified September 1st Greg Johnson
% to allow either interactive or parameter driven mode
% 
% (outlinefilename,outfilename,pitchflag,val,outputtype)
% 
%

% Usage:
% gen_outline(outlinefilename,outfilename,pitchflag,value,outputtype)
% THIS NEEDS EDITING!!
%
% Example:
% gen_outline('outline.txt','../fem/rail2','p',10,'patran')
%
% See http://www-2.cs.cmu.edu/~quake/triangle.html for documentation for
% the 'triangle' program
%
% Greg Johnson, July 2004

if nargin==0
  [outlinefilename,pathname]=uigetfile('*.txt','Select current boundary textfile');
  cd (pathname)
  outfilename=input('Name for resulting output file: ','s');
  pitchflag=input('Pitch (p) or Curvature (c) filtering (or none ()): ','s');
  val=input('Value for filtering (mm or deg (use zero for no filtering)): ');
  outputtype=input('Format for output file (patran, [dxf],poly): ','s');
if isempty(outputtype)
outputtype='dxf';
end

elseif nargin ~=5
  help gen_outline
  return
else
  outlinefilename=varargin{1};
  outfilename=varargin{2};
  pitchflag=varargin{3};
  val=varargin{4};
  outputtype=varargin{5};
end

% read the outline file first
[outline_orig.x,outline_orig.y,outline_orig.z]=read_three_columns(outlinefilename);

clf
% first decimate the outline by pitch or curvature

if upper(pitchflag)=='P'
  fprintf('Pitch filtering with pitch of %3.3f\n',val)
  outline=filter_pitch(outline_orig,val);
elseif upper(pitchflag)=='C'
  fprintf('Curvature filtering\n')
  outline=filter_curve(outline_orig,val);
elseif upper(pitchflag)=='N'
  disp('No filtering')
  outline=outline_orig;
else
  disp('Horrible problem with pitchflag!! - should be p or c')
  return
end


%%%%%%%
% for quad meshing, MUST have even number of points.
% so we nobble the second last point (arbitrary choice)
if mod(length(outline.x),2)
  outline.x(end-1)=[];
  outline.y(end-1)=[];
end
fprintf('Number of points: %d\n',length(outline.x));
x=outline.x;
y=outline.y;

plot(x,y,'.-')
axis equal
hold on
plot(x(diff(x)==0 & diff(y)==0),y(diff(x)==0 & diff(y)==0),'ro')
shg

if strcmp(upper(outputtype),'TRIANGLE')
  disp('Writing TRIANGLE format output')
  fid=fopen([outfilename '.poly'],'wt');
  fprintf(fid,'# auto-generated from Matlab\n');
  fprintf(fid,'0 2 0 0 \n');
  fprintf(fid,'# Boundary segments\n');
  fprintf(fid,'%d 1\n',length(x));
  for counter=1:length(x)-1
    fprintf(fid,'%d %d %d 1\n',counter,counter,counter+1);
  end
  fprintf(fid,'%d %d %d 1\n',counter+1,counter+1,1);
  fprintf(fid,'# Number of holes\n');
  fprintf(fid,'0\n');
  fclose(fid);

  fid=fopen([outfilename '.node'],'wt');
  fprintf(fid,'%d 2 0 0\n',length(x));
  for counter=1:length(x)
    fprintf(fid,'%d %f %f\n',counter,x(counter),y(counter));
  end
  fclose(fid);
  % -Pqa20
  cmd=sprintf('!triangle -q %s',[outfilename '.poly'])
  disp('Sizing can be changed...')
  eval(cmd);

elseif strcmp(upper(outputtype),'PATRAN')

  % x=outline_orig.x;
  % y=outline_orig.y;
  disp('Writing PATRAN format output')
  fid=fopen([outfilename '.out'],'wt');
  fprintf(fid,'25       0       0       1       0       0       0       0       0\n');
  fprintf(fid,'PATRAN neutral file generated by Matlab script (Greg Johnson)\n');
  fprintf(fid,'26       0       0       1       0       0       0       0       0\n');
  % date, time, version - should be fixed
  fprintf(fid,'17--04       10:10:38       3.0\n');

  % write 'grid data' (nodes of boundary)
  for counter=1:length(x)

    fprintf(fid,'31%8d%8d%8d%8d%8d%8d%8d%8d\n',counter,0,1,0,0,0,0,0);
    fprintf(fid,'%16.7E%16.7E%16.7E\n',x(counter),y(counter),0);
  end


  for counter=1:length(x)-1
    linecoef=[x(counter) x(counter+1) x(counter+1)-x(counter) x(counter+1)-x(counter);...
      y(counter) y(counter+1) y(counter+1)-y(counter) y(counter+1)-y(counter);...
      0 0 0 0]';

    fprintf(fid,'32%8d       0       3       0       0       0       0       0\n',counter);
    fprintf(fid,'%16.7E%16.7E%16.7E%16.7E%16.7E\n',linecoef(1:5));
    fprintf(fid,'%16.7E%16.7E%16.7E%16.7E%16.7E\n',linecoef(6:10));
    fprintf(fid,'%16.7E%16.7E',linecoef(11:12));
    fprintf(fid,'%8d%8d\n',counter,counter+1);
  end

  % write last segment manually
  linecoef=[x(end) x(1) x(1)-x(end) x(1)-x(end);...
    y(end) y(1) y(1)-y(end) y(1)-y(end);...
    0 0 0 0]';
  fprintf(fid,'32%8d       0       3       0       0       0       0       0\n',length(x));
  fprintf(fid,'%16.7E%16.7E%16.7E%16.7E%16.7E\n',linecoef(1:5));
  fprintf(fid,'%16.7E%16.7E%16.7E%16.7E%16.7E\n',linecoef(6:10));
  fprintf(fid,'%16.7E%16.7E',linecoef(11:12));
  fprintf(fid,'%8d%8d\n',length(x),1);

  fprintf(fid,'99       0       0       1       0       0       0       0       0');
  fclose(fid);

elseif strcmp(upper(outputtype),'DXF')
  disp('Writing DXF format output')
  fid=fopen([outfilename '.dxf'],'wt');
  fprintf(fid,'0\nSECTION\n2\n');
  fprintf(fid,'ENTITIES\n0\n');
  for counter=1:length(x)-1
    fprintf(fid,...
      'LINE\n10\n%f\n20\n%f\n30\n%f\n11\n%f\n21\n%f\n31\n%f\n0\n',...
      x(counter),y(counter),0,x(counter+1),y(counter+1),0);
  end
  % last line...
  fprintf(fid,...
    'LINE\n10\n%f\n20\n%f\n30\n%f\n11\n%f\n21\n%f\n31\n%f\n0\n',...
    x(counter+1),y(counter+1),0,x(1),y(1),0);

  fprintf(fid,'ENDSEC\n0\nEOF\n');
  fclose(fid);

else
  disp('Unknown output type')
end


if nargout>0
  varargout{1}=x;
  varargout{2}=y;
end
if nargin==0
  disp('')
  disp(['You can run ' mfilename ' as a function - like this:']);

  cmd=sprintf('%s(''%s'',''%s'',''%s'',%f,''%s'')',...
    mfilename,...
    outlinefilename,...
    outfilename,...
    pitchflag,...
    val,...
    outputtype);

  disp(cmd)
end


function deg=fixangle(radians);

deg=radians*180/pi;

if deg<0
  deg=deg+360;
end

deg=deg+360;


function outline=filter_pitch(outline_orig,val);
dist=0;
current.x=outline_orig.x(1);
current.y=outline_orig.y(1);
old=current;
prev=current;
outline.x=[];
outline.y=[];

if val==0
  outline.x=outline_orig.x;
  outline.y=outline_orig.y;
else
  for n=1:length(outline_orig.x)

    dist=dist+distance([prev.x;prev.y],[outline_orig.x(n);outline_orig.y(n)]);
    prev.x=outline_orig.x(n);
    prev.y=outline_orig.y(n);
    if dist>val
      if dist>2*val
        % fill in really long gaps
        if ~isempty(outline.x)
          xrange=linspace(outline.x(end),outline_orig.x(n),1+ceil(dist/val));
          yrange=linspace(outline.y(end),outline_orig.y(n),1+ceil(dist/val));
        else
          xrange=linspace(outline_orig.x(n-1),outline_orig.x(n),1+ceil(dist/val));
          yrange=linspace(outline_orig.y(n-1),outline_orig.y(n),1+ceil(dist/val));
        end
        outline.x=[outline.x xrange(2:end-1)];
        outline.y=[outline.y yrange(2:end-1)];

      end

      dist=0;
      old=current;
      outline.x=[outline.x outline_orig.x(n)];
      outline.y=[outline.y outline_orig.y(n)];
      plot(outline.x,outline.y,'r.-')
      axis equal
      drawnow
    end
  end
  plot(outline.x,outline.y,'r.-');
  axis equal
  drawnow
end

function outline=filter_curve(outline_orig,val)
% curvature based filtering

orig_length=length(outline_orig.x);

outline.x=[];
outline.y=[];
curveflag=0;
range=round(orig_length/40);
init=1;

for n=1:orig_length
  if n+range > orig_length
    % wrap around if necessary
    segment.x=[outline_orig.x(n:end); outline_orig.x(1:range-(orig_length-n))];
    segment.y=[outline_orig.y(n:end); outline_orig.y(1:range-(orig_length-n))];
  else
    segment.x=[outline_orig.x(n:n+range)];
    segment.y=[outline_orig.y(n:n+range)];
  end

  xA1=segment.x(1);   yA1=segment.y(1);
  xA2=segment.x(end); yA2=segment.y(end);
  
  if curveflag~=0 | init==1
    init=0;
    yB1=segment.y(1);
    yB2=segment.y(round(end/2));
    xB1=segment.x(1);
    xB2=segment.x(round(end/2));
  end

  theta1=fixangle(atan2(yA2-yA1,xA2-xA1));
  theta2=fixangle(atan2(yB2-yB1,xB2-xB1));

  dt1=abs(theta1-theta2);
  dt2=abs(dt1-360);
  difftheta=min([dt1 dt2]);

  if difftheta>val
    curveflag=round(orig_length/50);  % was /50
    % if angle > threshold specified by user
    outline.x=[outline.x xB1];
    outline.y=[outline.y yB1];
    plot(xB1,yB1,'ro')
    hold on
  
  elseif curveflag>1
    % add a few extra points after things go flat again
    curveflag=curveflag-1;
    outline.x=[outline.x xA1];
    outline.y=[outline.y yA1];
    
  end
  
end

if curveflag==0
  % in straight section as we finished
  % delete first point
  outline.x(1)=[];
  outline.y(1)=[];
  % connect first and last points
  %   outline.x(end+1)=outline.x(1);
  %   outline.y(end+1)=outline.y(1);
end
