function cmm=gen_outline(cmm,outfilename,pitchflag,val)
% GEN_OUTLINE takes cmm boundary data (twin column text file) and 
% produces a DXF file for Abaqus to use...
% Usage:
% gen_outline(cmm,outfilename,pitchflag,val)
% where cmm is the dataset with components cmm.xAout, cmm.yAout,cmm.zAout
% outfilename is the name of the DXF file to be written
% pitchflag is 'p' or 'c' for fixed Pitch or variable Curvature
% val is the pitch (mm) or the curvature (degrees)
  
  if nargin~=4
    disp('Not enough arguments...');
    help gen_outline
    return
  end
  
  disp('Working in mm...');

  
  cmm.bx=cmm.xAout;
  cmm.by=cmm.yAout;
  cmm.bz=cmm.zAout;
  

  if upper(pitchflag)=='P'
    filename=outfilename;
    fid=fopen(filename,'wt');
    fprintf(fid,'0\nSECTION\n2\n');
    fprintf(fid,'ENTITIES\n0\n');
    disp(['Writing ' filename]);
    
    dist=0;
    current.x=cmm.bx(1);
    current.y=cmm.by(1);
    current.z=cmm.bz(1);
    old=current;
    clf
    
    prev=current;
    
    cmm.newx=[];
    cmm.newy=[];
    for n=1:length(cmm.bx)
      dist=dist+distance([prev.x;prev.y],[cmm.bx(n);cmm.by(n)]);
      prev.x=cmm.bx(n);
      prev.y=cmm.by(n);
      
      if dist>val
        dist=0;
        old=current;
        current.x=cmm.bx(n);
        current.y=cmm.by(n);
        fprintf(fid,...
                'LINE\n10\n%f\n20\n%f\n30\n%f\n11\n%f\n21\n%f\n31\n%f\n0\n',...
                old.x,...
                old.y,...
                old.z,...
                current.x,...
                current.y,...
                current.z);
        plot(current.x,current.y,'.');
        cmm.newx=[cmm.newx current.x];
        cmm.newy=[cmm.newy current.y];
        hold on
      end
    end
    
  fprintf(fid,'LINE\n10\n%f\n20\n%f\n30\n%f\n11\n%f\n21\n%f\n31\n%f\n0\n',...
          current.x,...
          current.y,...
          current.z,...
          cmm.bx(1),...
          cmm.by(1),...
          cmm.bz(1));
  
  
  
  fprintf(fid,'ENDSEC\n0\nEOF\n');
  fclose(fid);
    
    
    
  elseif upper(pitchflag)=='C'
    if isfield(cmm,'newx')
      % if a pitch reduced variable exists, use it instead
      cmm.bx=cmm.newx;
      cmm.by=cmm.newy;
    end
    
    cmm.d=nan*ones(size(cmm.bx));
    cmm.cx=[];
    cmm.cy=[];
    curveflag=0;
    range=round(length(cmm.bx)/40);
    firstpoint=1;
    init=1;
    pix=0;
    if pix
      figure(1)
      clf
      plot(cmm.bx,cmm.by,'k.','markersize',1);
      hold on
      pA=plot(0,0,'go-');

      pB=plot(0,0,'bo-');
    end
    for n=1:length(cmm.bx)
      if n+range > length(cmm.bx)
        % wrap around if necessary
        segment.x=[cmm.bx(n:end) cmm.bx(1:range-(length(cmm.bx)-n))];
        segment.y=[cmm.by(n:end) cmm.by(1:range-(length(cmm.bx)-n))];
      else
        segment.x=[cmm.bx(n:n+range)];
        segment.y=[cmm.by(n:n+range)];
      end
    
      yA1=segment.y(1);
      yA2=segment.y(end);
      xA1=segment.x(1);
      xA2=segment.x(end);

      if curveflag~=0 | init==1
        init=0;
        yB1=segment.y(1);
        yB2=segment.y(round(end/2));
        xB1=segment.x(1);
        xB2=segment.x(round(end/2));
      end      

      if pix
        set(pA,'xdata',[xA1 xA2],'ydata',[yA1 yA2]);
        set(pB,'xdata',[xB1 xB2],'ydata',[yB1 yB2]);
        drawnow
        pause
      end
      
      
      theta1=fixangle(atan2(yA2-yA1,xA2-xA1));
      theta2=fixangle(atan2(yB2-yB1,xB2-xB1));

      dt1=abs(theta1-theta2);
      dt2=abs(dt1-360);
      difftheta=min([dt1 dt2]);
      cmm.d(n)=difftheta;
      
      if curveflag>0
        % add a few extra points after things go flat again
        curveflag=curveflag-1;
        cmm.cx=[cmm.cx xA1];
        cmm.cy=[cmm.cy yA1];
      end
      if difftheta>val
        curveflag=round(length(cmm.bx)/50)
        % if angle > threshold specified by user
        cmm.cx=[cmm.cx xB1];
        cmm.cy=[cmm.cy yB1];
      end
      
    end
    if curveflag==0
      % in straigh section as we finished
      % delete first point
      cmm.cx(1)=[];
      cmm.cy(1)=[];
      cmm.d(1)=[];
      % connect first and last points
      cmm.cx(end+1)=cmm.cx(1);
      cmm.cy(end+1)=cmm.cy(1);
    end

    
    figure(2)
    clf
    plot(cmm.bx,cmm.by,'y.');
    hold on
    plot(cmm.cx,cmm.cy,'b.-');
    axis equal

    
    
    
    
    filename=outfilename;
    fid=fopen(filename,'wt');
    fprintf(fid,'0\nSECTION\n2\n');
    fprintf(fid,'ENTITIES\n0\n');
    
    for n=1:length(cmm.cx)-1
      fprintf(fid,...
              'LINE\n10\n%f\n20\n%f\n30\n%f\n11\n%f\n21\n%f\n31\n%f\n0\n',...
              cmm.cx(n),...
              cmm.cy(n),...
              0,...
              cmm.cx(n+1),...
              cmm.cy(n+1),...
              0);
      
    end
  
  
  fprintf(fid,'ENDSEC\n0\nEOF\n');
  fclose(fid);

    
      

    
  else
    disp('Horrible problem with pitchflag!!')
    return
  end
  


  
function deg=fixangle(radians);
  
  deg=radians*180/pi;
  
  if deg<0
    deg=deg+360;
  end
  
  deg=deg+360;